use std::process::{Command, ExitStatus};

use colored::Colorize;
use log::{error, info};
use serde_derive::Deserialize;

use crate::result::Result;

/// structure to define an app config
#[derive(Clone, Debug, Deserialize)]
pub struct App {
	/// name of the app
	pub name: String,

	/// path/command to run
	pub command: String,

	/// arguments
	pub args: Option<Vec<String>>,

	/// should the app be restarted if it exits successfully
	pub restart_on_success: Option<bool>,

	/// should the app be restarted if it exits unsuccessfully
	pub restart_on_failure: Option<bool>,

	/// should the app be restarted if it is killed (SIGKILL)
	pub restart_on_terminate: Option<bool>,

	/// on sup startup, should we wait for this app to run before continuting
	pub wait: Option<bool>,

	/// should this app keep sup from exiting, if it's running
	pub hold: Option<bool>,
}

impl App {
	/// start the application and wait for it to exit
	pub fn wait_start(&self) -> Result<()> {
		info!("[{}] starting", &self.name.yellow());

		let mut command = Command::new(&self.command);
		let args = self.args.as_ref();
		command.args(args.unwrap_or(&Vec::new()));

		match command.output() {
			Ok(output) => {
				if !output.stdout.is_empty() {
					let s = String::from_utf8(output.stdout)?;
					for line in s.lines() {
						info!("[{}] stdout: {}", &self.name.yellow(), line);
					}
				}
				if !output.stderr.is_empty() {
					let s = String::from_utf8(output.stderr)?;
					for line in s.lines() {
						error!("[{}] stderr: {}", &self.name.yellow(), line);
					}
				}
				Ok(())
			}
			Err(e) => Err(Box::from(e)),
		}
	}

	/// see if the app should be restarted
	pub fn check_restart(&self, status: ExitStatus) -> bool {
		if status.success() {
			self.restart_on_success.unwrap_or(false)
		} else if let Some(_code) = status.code() {
			self.restart_on_failure.unwrap_or(false)
		} else {
			self.restart_on_terminate.unwrap_or(false)
		}
	}
}
